#include <fxengine/model/bitmap.h>

#include <gint/display.h>
#include <gint/std/string.h>
#include <gint/std/stdlib.h>

fe_bitmap* fe_bitmap_new(uint32_t size_px_x, uint32_t size_px_y, uint32_t* color, bool copy_color, uint32_t *layout, bool copy_layout)
{
    fe_bitmap* bmp = malloc(sizeof(fe_bitmap));
    if (!bmp)
        return 0;

    bmp->size_px_x = size_px_x;
    bmp->size_px_y = size_px_y;

    bmp->size_o_y = 1 + (size_px_x - 1)/(8*sizeof(uint32_t));

    bmp->color_dynamic = copy_color;
    bmp->layout_dynamic = copy_layout;

    uint32_t size_octets = size_px_y * bmp->size_o_y * sizeof(uint32_t);

    if (copy_color)
    {
        bmp->color = malloc(size_octets);
        if (!bmp->color)
        {
            free(bmp);
            return 0;
        }
        memcpy(bmp->color, color, size_octets);
    }
    else
        bmp->color = color;

    if (copy_layout && layout)
    {
        bmp->layout=malloc(size_octets);
        if (!bmp->layout)
        {

            free(bmp);
            if (copy_color)
                free(bmp->color);
            return 0;
        }
        memcpy(bmp->layout, layout, size_octets);
    }
    else
        bmp->layout = layout;

    return bmp;
}

void fe_bitmap_del(fe_bitmap * bmp)
{
    if (!bmp)
        return;

    if (bmp->layout_dynamic && bmp->layout)
        free(bmp->layout);

    if (bmp->color)
        free(bmp->color);

    free(bmp);
}

uint8_t fe_bitmap_get_px(const fe_bitmap * bmp, uint32_t x, uint32_t y)
{
    if (x >= bmp->size_px_x || y >= bmp->size_px_y)
        return 0;

    const uint32_t indice = y * bmp->size_o_y + (x / 32);

    const uint32_t numero_bit = 31 - (x %32);

    if (bmp->layout)
        return  (((bmp->layout[indice] & (1 << numero_bit)) >> numero_bit) + ((bmp->color[indice] & (1 << numero_bit)) >> numero_bit));
    else
        return (0b10 + ((bmp->color[indice] & (1 << numero_bit)) >> numero_bit));
}


void fe_bitmap_display_px(const fe_bitmap * bmp, uint32_t bmp_x, uint32_t bmp_y, uint32_t scr_x, uint32_t scr_y)
{
    uint8_t color = fe_bitmap_get_px(bmp, bmp_x, bmp_y);
    if (color)
        dpixel(scr_x, scr_y, 3 * (color % 2));
}
